package FFRIndexed::Template;
use strict;
use FFRIndexed;
use Table::Denormalized::ColumnRefiner;
use base 'Class::Accessor';
use Data::Dumper;
use Data::Page;
use CGI;
use HTML::Prototype;
use CGI::PersistentParams;

use Memoize;
use Module::Pluggable::Fast
  name   => '_plugins',
  ;

memoize('plugins');

use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

FFRIndexed::Template - Render and interactively refine a FFRIndexed query (name to change)

A likely good name would maybe be FFRIndexed::View
or something like that, as it's a class conveniently usable
from any templating system that has method calls on objects.

=cut

__PACKAGE__->mk_accessors(qw(
  query
  index refiner order
  root parts
  page items_per_page
  template

  persistent_params

  ajax
));

=head2 C<< __PACKAGE__->new ARGS >>

Creates a new view. The following
parameters are optional but allowed:

=head3 C<index>

The L<FFRIndexed> file index to use. If none is given,
one will be instantiated.

=head3 C<query>

The query (as data structure) to use. Default
is C<[]>.

=head3 C<default_category>

The default category to use.

=head3 C<columns>

The columns to display.

=head3 C<order>

The order in which the columns are displayed.

=head3 C<refiner>

An instance of L<Table::Denormalized::ColumnRefiner>

=head3 C<persistent_params>

List of the parameters which persist over queries. This
is used to append/discard CGI parameters from
the query string. If none is given, a new instance
of L<CGI::PersistentParams> is created with the following
columns to discard:

  action _id super_search Search column template ss _

=head3 C<items_per_page>

The number of items per page. Default is 20.

=head3 C<page>

The page to display. Default is 0.

=head3 C<pager>

The pager object to use. Default is
a new instance of L<Data::Page>

=head3 C<ajax>

The L<HTML::Prototype> object to use. Default
is a new instance.

=cut

sub new {
  my ($package,%args) = @_;
  $args{index} ||= FFRIndexed->new();

  $args{query} ||= [];

  $args{default_category} ||= "artist";

  my $default = delete $args{default_category};

  $args{columns} ||= [ map { $_->name } $args{index}->columns ];
  $args{order}   ||= [ @{$args{columns}} ];
  $args{refiner} ||= Table::Denormalized::ColumnRefiner->new(
          columns => delete $args{columns},
          table => $args{index} );

  $args{persistent_params} ||= CGI::PersistentParams->new(
    discard => [qw[action _id super_search Search column template ss _]],
  );

  $args{items_per_page} ||= 20;
  $args{page}  ||= 0;
  $args{pager} ||= Data::Page->new();
  $args{ajax} ||= HTML::Prototype->new();

  $package->SUPER::new(\%args);
};

=head2 C<< $view->link $ARGREF >>

Returns a link to the current query, modified
by C<$ARGREF>, suitable for use from
within Template Toolkit. Very convenient if you want to page
through a sorted set. All CGI query parameters
are preserved. If you want to erase a parameter,
pass <tt>undef</tt> in the argument.

=cut

sub link {
  my ($self,$args) = @_;
  $self->persistent_params->link($args);
};

=head2 C<< $view->hidden $ARGREF >>

Returns a list of hidden parameters of
the current query from C<persistent_params>,
as modified by C<$ARGREF>. Convenient
for inclusion in a form.

=cut

sub hidden {
  my ($self,$args) = @_;
  $self->persistent_params->hidden($args);
};

=head2 C<< $view->plugins TYPE >>

Returns the list of available plugins
of the given type. The type can be one of

  page
  ...
  ...

=cut

sub plugins {
  my ($self,$type) = @_;
  grep { exists ${$_->types}{$type} && $_->visual } $self->_plugins;
};

=head2 C<< $view->refinements >>

Returns the list of refinements for the current query.

=cut

sub refinements {
  my ($self) = @_;
  # Move to Memoize?

  $self->{refinements} ||= [ $self->refiner->refinements( $self->query ) ];

  @{ $self->{refinements} };
};

=head2 C<< $view->implicit_columns >>

Returns the list of the columns that were not
explicitly named in the query, but still add no
subdivision to the results.

=cut

sub implicit_columns {
  my ($self) = @_;
  # Move to Memoize?

  # Now, remove the explicit columns as given in the
  # (top level of the) query:
  my %q = map { $_ => 1 } $self->explicit_columns;
  $self->{implicit_columns} ||= [ grep {! exists $q{$_}}
                                  $self->refiner->implicit_columns( $self->query ) ];

  @{ $self->{implicit_columns} };
};

=head2 C<< explicit_columns >>

Returns the names of the columns mentioned explicitly
in the query. This needs to be set in the constructor,
otherwise some dodgy and untested logic will be
used that will likely mistreat SuperSearches, as these
potentially mention many columns.

=cut

sub explicit_columns {
  my ($self) = @_;
  # Move to Memoize?

  #warn "Getting explicit columns";
  #$self->{explicit_columns} ||= [ sort queried_columns($self->query) ];
  #$self->{explicit_columns} = [ queried_columns($self->query) ];
  # warn Dumper $self->{explicit_columns};
  @{ $self->{explicit_columns} };
};

=head2 C<< $view->refinement $COLUMN >>

Returns the refinement for the column C<$COLUMN>

=cut

sub refinement {
  my ($self,$column) = @_;
  my ($res) = grep { $_->column eq $column } $self->refinements;
  $res;
};

=head2 C<< $view->refining_columns >>

Returns the refining columns for the current query.

=cut

sub refining_columns {
  my ($self) = @_;
  $self->refinements;
};

=head2 C<< $view->columns >>

Returns all columns

=cut

sub columns { shift->refiner->columns };

=head2 C<< $view->interesting_columns >>

Returns all non-trivial columns from
the set of given columns. Using this should
somehow differentiate from user-requested
columns and determined columns. Some day.

Maybe "refining columns" should take a list
of admissible columns. The the two functions
could be merged into one function.

=cut

sub interesting_columns {
  my ($self) = @_;
  my %r = map { $_ => 1 } ($self->implicit_columns, $self->explicit_columns);
  grep { ! exists $r{$_} } @{ $self->columns }
};

=head2 C<< $view->resultset START, COUNT >>

Initializes and retrieves the full resultset
from the database. It returns the resultset
as an array reference.
This is currently overkill
as only the overall size of the resultset and
the items on the "current page" are actually
needed.

This method will later return only the count
and the "relevant" rows and not retrieve
everything from the database.

Or maybe not, as the Playlist plugin relies
on the fact that all rows are available.

=cut

sub resultset {
  my ($self) = @_;

  my $page  = $self->page;
  my $count = $self->items_per_page;

  # Later on, we should maybe do the splice
  # via a LIMIT clause instead of
  # splicing after the fact:
  $self->{resultset} ||= $self->index->query_ex(
    where => $self->query,
    order => $self->order
  );

  my $pager = $self->{pager};

  $pager->total_entries(scalar @{ $self->{resultset} });
  $pager->entries_per_page($count);
  $pager->current_page($page);

  #warn sprintf "%s/%s/%s (%s)", $pager->first_page, $pager->current_page, $pager->last_page, $count;

  $self->{resultset};
};

=head2 C<< $view->resultset_list START, COUNT >>

Same as C<resultset>, except that it returns a list
instead of an array reference.

=cut

sub resultset_list {
  my $self = shift;
  @{ $self->resultset(@_) }
};

=head2 C<< $view->current_content >>

Returns the next C<COUNT> items after C<START>.
This is convenient to implement the concept of
a "current page", for example using L<Data::Pager>.

=cut

sub current_content {
  my ($self) = @_;

  my $count = $self->items_per_page;
  my $page = $self->page;
  #warn "Getting current content ($page/$count)";

  my $resultset = $self->resultset;
  my $pager = $self->pager();

  $pager->splice( $self->{resultset} );
};

=head2 C<< $view->pager >>

Returns a L<Data::Page> object for convenient
paging through the resultset.

=cut

sub pager {
  my ($self) = @_;
  # prime the pager
  $self->resultset;
  $self->{pager};
};

=head2 C<< $view->keyword_complete KEYWORD COLUMNS >>

Returns a (unique) list of matching keywords
from the C<current_content>.

The matching is done via case insensitive
substring matching.

This should likely go somewhere else separate
instead of being part of this module.

=cut

sub keyword_complete {
  my ($self,$keyword,@columns) = @_;

  # Unwrap all array references (from TT)
  @columns = map { ref $_ ? @$_ : $_ } @columns;

  @columns = $self->interesting_columns
    unless @columns;
  my %result;

  my $kw = uc $keyword;
  for my $r (@{$self->resultset}) {
    # XXX Make the keyword columns configurable
    for my $m (@columns) {
      my $v = $r->$m;
      # warn "$kw - $v";
      my $e = uc $v;
      $result{$e} = $v
        if (index $e, $kw) >= 0;
    }
  };

  sort { uc $a cmp uc $b } values %result
};

1;
